﻿#region Using directives

using System;
using System.Collections.Generic;

#endregion

namespace STL.Generic
{
    public sealed class Vector<T>
    {
        sealed class VectorIterator : IRandomIterator<T>
        {
            Vector<T> vector;
            private int position;

            public VectorIterator(Vector<T> v) : this(v, 0) { }
            public VectorIterator(VectorIterator it) : this(it.vector, it.position) { }

            public VectorIterator(Vector<T> v, int pos)
            {
                vector = v;
                position = pos;
            }

            public T Value
            {
                get { return vector[Position]; }
                set { vector[Position] = value; }
            }

            public IForwardIterator<T> Next()
            {
                ++position;
                return this;
            }

            public IBidirectionalIterator<T> Prev()
            {
                --position;
                return this;
            }

            public int Position
            {
                get { return position; }
                set { position = value; }
            }

            public IRandomIterator<T> Offset(int n)
            {
                Position += n;
                return this;
            }

            public bool Equals(VectorIterator obj)
            {
                return Position == obj.Position;
            }

            public override bool Equals(object obj)
            {
                return Equals((VectorIterator)obj);
            }

            public override int GetHashCode()
            {
                return base.GetHashCode();
            }

            public object Clone()
            {
                return base.MemberwiseClone();
            }
        }

        const int InitialSize = 16;
        private T[] data;
        private int size;
        private IRandomIterator<T> First;
        private IRandomIterator<T> Last;

        public Vector() : this(0) { }

        public Vector(int n)
        {
            data = new T[n];
            size = n;
            First = new Vector<T>.VectorIterator(this, 0);
            Last = new Vector<T>.VectorIterator(this, size);
        }

        public void Reserve(int n)
        {
            if (n > Capacity)
            {
                T[] a = new T[n];
                Array.Copy(data, a, Size);
                data = a;
            }
        }

        public int Capacity
        {
            get { return data.Length; }
        }

        public IRandomIterator<T> Begin()
        {
            return new VectorIterator(this, 0);
        }

        public IRandomIterator<T> End()
        {
            return new VectorIterator(this, Size);
        }

        public IForwardIterator<T> Rbegin()
        {
            return new ReverseIterator<T>(new VectorIterator(this, Size - 1));
        }

        public IForwardIterator<T> Rend()
        {
            return new ReverseIterator<T>(new VectorIterator(this, -1));
        }

        public int Size
        {
            get { return size; }
        }

        public bool Empty()
        {
            return Size == 0;
        }

        public T At(int index)
        {
            if (index < 0 || index >= Size)
                throw new IndexOutOfRangeException();
            return data[index];
        }

        public T this[int index]
        {
            get { return data[index]; }
            set { data[index] = value; }
        }

        public T Front
        {
            get { return data[0]; }
            set { data[0] = value; }
        }

        public T Back
        {
            get { return data[Size - 1]; }
            set { data[Size - 1] = value; }
        }

        public void PushBack(T x)
        {
            if (Size >= Capacity)
            {
                Reserve(Capacity * 2 + 1);
            }
            data[size++] = x;
            Last.Position = Size - 1;
        }

        public IRandomIterator<T> Erase(IRandomIterator<T> p)
        {
            IRandomIterator<T> q = (IRandomIterator<T>)p.Clone();
            Algorithm.Copy<T>(q.Next(), Last, p);
            Destroy(new VectorIterator(this, Size - 1), Last);
            Last.Position = --size;
            return p;
        }

        public IRandomIterator<T> Erase(IRandomIterator<T> first, IRandomIterator<T> last)
        {
            IForwardIterator<T> i = Algorithm.Copy<T>(last, Last, first);
            Destroy(i, Last);
            Last.Position = size = first.Position;
            return first;
        }

        public IRandomIterator<T> Clear()
        {
            return Erase(Begin(), Last);
        }

        private void Destroy(IForwardIterator<T> first, IForwardIterator<T> last)
        {
            while (!first.Equals(last))
            {
                first.Value = default(T);
                first.Next();
            }
        }
    }
}